#!/bham/pd/bin/php -q
<?php

	define('JCJ_TEMPLATE_FILENAME', 'jcj.template');
	define('TMP_DIRECTORY', '/tmp');
	define('COMMAND_PROVERIF', '/sw/bin/proverif -in pi');
	define('COMMAND_EGREP', 'egrep');

	define('STAT_MODE', true);
	define('STAT_MODE_RAW', true);

	if ($argc != 2 || in_array($argv[1], array('--help', '-help')) || ((int)$argv[1]) <= 1) {
		echo "This script enables the analysis of the JCJ protocol with respect to a given length of candidate list\n";
		echo "\n";
		echo "Usage: " . $argv[0] . " <candidate list size>\n";
		echo "\n";
		echo "Where <candidate list size> must be an integer greater than 1\n";
		die();
	}

	$candidates = $argv[1];

	$expectedResult = array('RESULT not ev:pass(true()) is false.',
							'RESULT not ev:fail(true()) is true.');

	$handle = fopen(JCJ_TEMPLATE_FILENAME, 'r');
	$template = fread($handle, filesize(JCJ_TEMPLATE_FILENAME));
	fclose($handle);

	$errorCount = 0;
	$startTime = time();
	$statTime = array();

	echo "STARTING ANALYSIS. ProVerif is analysing the security of JCJ with respect to 1-$candidates candidates.\n";
	echo "Expected output:\n";
	for ($i = 0; $i < count($expectedResult); $i++) {
		echo "  " . $expectedResult[$i] . "\n";
	}
	echo "=====================================\n";	

	for ($i = 1; $i <= $candidates; $i++) {
		$filename = tempnam(TMP_DIRECTORY, 'jcj');
		$handle = fopen($filename, 'w');
		fwrite($handle, make_proverif($i, $template));
		fclose($handle);
	
		echo "STARTING ANALYSIS. ProVerif is analysing the security of JCJ with respect to $i candidates. ($filename)\n";

		$tmpTime = time();
		
		unset($shellResult);
		exec(COMMAND_PROVERIF . " $filename | " . COMMAND_EGREP . ' "RES|War|Err|File"', $shellResult);

		$statTime[$i] = time() - $tmpTime;

		for ($j = 0; $j < count($shellResult); $j++) {
			echo "  " . $shellResult[$j] . "\n";
		}

		if (count(array_diff($shellResult,$expectedResult)) == 0) {
			echo "ANALYSIS SUCCESS. ProVerif has verified the security of JCJ with respect to $i candidates.\n";
		} else {
			$errorCount++;
			echo "ANALYSIS ERROR. This script has been unable to verify the security of JCJ with respect to $i candidates.\n";
		}

		echo "=====================================\n";
	}

	if ($errorCount == 0) {
		echo "ANALYSIS COMPLETE. ProVerif has verified the security of JCJ with respect to 1-$candidates candidates.\n";
	} else {
		echo "ERROR(S) OCCURRED. This script has been unable to verify the security of JCJ with respect to 1-$candidates candidates.\n";
	}

	echo "ELAPSE TIME: " . time_duration(time() - $startTime) . "\n";

	if (STAT_MODE) {
		echo "Breakdown of candidate runs:\n";

		for ($i = 1; $i <= count($statTime); $i++) {
			echo "$i\t\t";
			echo (STAT_MODE_RAW) ? $statTime[$i] : time_duration($statTime[$i]);
			echo "\n";
		} 
	}



    function make_proverif($candidates, $template) {
    	$candidateNames = '';
    	$candidateVars = '';
    	$equationZkver1 = '';
    	$processR = '';

    	for ($i = 1; $i <= $candidates; $i++) {
    		$candidateNames .=  "n$i,";
    		$candidateVars .=  'z1_' . ($i+3) . ',';
    		$processR .= "(!out(b,n$i)) | " ;

    		$equationZkver1 .= "equation zkver1(zk1((v,r1,k,r2),(penc(v,r1,pky),penc(k,r2,pky),pky";

    		for ($j = 1; $j <= $candidates; $j++) {
    			$equationZkver1 .= ($i == $j) ? ',v' : ",z$j";
    		}

    		$equationZkver1 .= "))) = true.\n";
    	}
    	
    	$candidateNames = substr($candidateNames,0,-1);
    	$candidateVars = substr($candidateVars,0,-1);
    	$processR = substr($processR,0,-3) . ".";


		$template = str_replace('[[:CANDIDATE_NAMES:]]', $candidateNames, $template);
		$template = str_replace('[[:CANDIDATE_VARS:]]', $candidateVars, $template);
		$template = str_replace('[[:PROCESS_R:]]', $processR, $template);
		$template = str_replace('[[:EQUATIONS_ZKVER1:]]', $equationZkver1, $template);

		return $template;
    }


    /**
     * A function for making time periods readable
     *
     * @author      Aidan Lister <aidan@php.net>
     * @version     2.0.0
     * @link        http://aidanlister.com/2004/04/making-time-periods-readable/
     * @param       int     number of seconds elapsed
     * @param       string  which time periods to display
     * @param       bool    whether to show zero time periods
     */
    function time_duration($seconds, $use = null, $zeros = false)
    {
        // Define time periods
        $periods = array (
            'years'     => 31556926,
            'Months'    => 2629743,
            'weeks'     => 604800,
            'days'      => 86400,
            'hours'     => 3600,
            'minutes'   => 60,
            'seconds'   => 1
            );

        // Break into periods
        $seconds = (float) $seconds;
        foreach ($periods as $period => $value) {
            if ($use && strpos($use, $period[0]) === false) {
                continue;
            }
            $count = floor($seconds / $value);
            if ($count == 0 && !$zeros) {
                continue;
            }
            $segments[strtolower($period)] = $count;
            $seconds = $seconds % $value;
        }

        // Build the string
        foreach ($segments as $key => $value) {
            $segment_name = substr($key, 0, -1);
            $segment = $value . ' ' . $segment_name;
            if ($value != 1) {
                $segment .= 's';
            }
            $array[] = $segment;
        }

        $str = implode(', ', $array);
        return $str;
    }
?>
